import typing

from pydantic import Field
from pydantic_settings import BaseSettings, SettingsConfigDict

from ._version import __version__


class Configuration(BaseSettings):
    """Configuration for the Magicbell API.

    Values can be set programmatically by constructing a `Configuration` object;
    or they can be set as environment variables prefixed with `MAGICBELL_`.

    Examples
    --------
    >>> import os
    >>> from belfry_magicbell.configuration import Configuration
    >>> config = Configuration(api_key="my-api-key", api_secret="my-api-secret")
    >>> os.environ["MAGICBELL_API_KEY"] = "my-api-key"
    >>> os.environ["MAGICBELL_API_SECRET"] = "my-api-secret"
    >>> env_config = Configuration()
    >>> config == env_config
    """

    api_url: str = Field(
        "https://api.magicbell.com", description="The base URL for the Magicbell API."
    )
    api_key: typing.Optional[str] = Field(
        description="API key for the Magicbell API, sent as `X-MAGICBELL-API-KEY` header.",
        default=None,
    )
    api_secret: typing.Optional[str] = Field(
        description="API secret for the Magicbell API, sent as `X-MAGICBELL-API-SECRET` header.",
        default=None,
    )

    user_jwt: typing.Optional[str] = Field(
        description="A MagicBell user JWT to access protected resources such as projects..",
        default=None,
    )

    request_timeout_seconds: float = Field(
        5.0,
        description="The timeout for requests. "
        "An exception will be raised if the request takes longer than this.",
    )
    model_config = SettingsConfigDict(env_prefix="magicbell_")

    def get_user_external_id_headers(self, external_id: str):
        """
        Return headers to use when the users id is needed to be included in the request
        """
        headers = self.get_general_headers()
        headers["x-magicbell-user-external-id"] = external_id
        return headers

    def get_general_headers(
        self, idempotency_key: typing.Optional[str] = None
    ) -> typing.Dict[str, str]:
        """Return headers to use for non-project related requests.
        These headers are most often used.

        This includes `self.api_key` as `X-MAGICBELL-API-KEY` and `self.api_secret` as `X-MAGICBELL-API-SECRET`.
        """
        headers = self.get_base_headers()
        if idempotency_key:
            headers["IDEMPOTENCY-KEY"] = idempotency_key
        if self.api_key:
            headers["X-MAGICBELL-API-KEY"] = self.api_key
        if self.api_secret:
            headers["X-MAGICBELL-API-SECRET"] = self.api_secret
        return headers

    def get_user_headers(
        self, idempotency_key: typing.Optional[str] = None
    ) -> typing.Dict[str, str]:
        """Return headers to use when a user JWT is required, such as projects.

        This includes `self.user_jwt` as `Authorization: Bearer <JWT>`.
        """
        headers = self.get_base_headers()
        if idempotency_key:
            headers["IDEMPOTENCY-KEY"] = idempotency_key
        if self.user_jwt:
            headers["Authorization"] = f"Bearer {self.user_jwt}"
        return headers

    def get_base_headers(self) -> typing.Dict[str, str]:
        """Return headers that should be used for all requests."""
        return {
            "User-Agent": f"magicbell-python/{__version__}",
            "Content-Type": "application/json",
            "Accept": "application/json",
        }



from .channels import ChannelsAPI  # noqa: F401
from .graphql import GraphQLAPI  # noqa: F401
from .project import ProjectAPI  # noqa: F401
from .push_subscription import PushSubscriptionAPI  # noqa: F401
from .realtime import RealtimeAPI  # noqa: F401
from .user import UserAPI  # noqa: F401



import typing

from ..model.push_subscription import WrappedPushSubscription
from ..model.response import Response
from ._base import BaseAPI
from ._parsing import build_request_content, build_response


class PushSubscriptionAPI(BaseAPI):
    async def create_push_subscription(
        self,
        external_id: str,
        wrapped_push_subscription: typing.Union[WrappedPushSubscription, typing.Dict],
    ) -> WrappedPushSubscription:
        """
        Register a device token for push notifications.

        Ref: https://www.magicbell.com/docs/rest-api/reference#push-subscriptions-create
        """
        response = await self.client.post(
            "/push_subscriptions",
            headers=self.configuration.get_user_external_id_headers(external_id=external_id),
            content=build_request_content(wrapped_push_subscription),
        )
        return build_response(response=response, out_type=WrappedPushSubscription).parsed

    async def delete_push_subscription(
        self, external_id: str, device_token: str
    ) -> Response[typing.Type[None]]:
        """
        Deletes the registered device token to remove the mobile push subscription.

        Ref: https://www.magicbell.com/docs/rest-api/reference#push-subscriptions-delete
        """
        url = f"/push_subscriptions/{device_token}"
        response = await self.client.delete(
            url, headers=self.configuration.get_user_external_id_headers(external_id=external_id)
        )
        return build_response(response=response, out_type=None)



from .channels import Channel, ChannelConfiguration, WrappedChannels  # noqa: F401
from .notification import (  # noqa: F401
    ChannelOverrides,
    CreatedNotificationBroadcast,
    Notification,
    NotificationChannelsOverrides,
    NotificationOverrides,
    NotificationProvidersOverrides,
    Recipient,
    WrappedCreatedNotificationBroadcast,
    WrappedNotification,
)
from .project import (  # noqa: F401
    Project,
    ProjectInput,
    WrappedProject,
    WrappedProjectInput,
    WrappedProjects,
)
from .push_subscription import PushSubscription, WrappedPushSubscription  # noqa: F401
from .response import Response, ResponseBodyT  # noqa: F401
from .user import User, WrappedUser  # noqa: F401
from .workspace import Workspace  # noqa: F401



import typing

from ..model.response import Response
from ..model.user import WrappedUser
from ._base import BaseAPI
from ._parsing import build_request_content, build_response


class UserAPI(BaseAPI):
    async def create_user(
        self,
        wrapped_user: typing.Union[WrappedUser, typing.Dict],
        idempotency_key: typing.Optional[str] = None,
    ) -> WrappedUser:
        """Create a user, returning a `WrappedUser`.
        Please note that you must provide the user's email or the external id so MagicBell can uniquely identify the user.
        The external id, if provided, must be unique to the user.
        """
        return (await self.create_user_detailed(wrapped_user, idempotency_key)).parsed

    async def create_user_detailed(
        self,
        wrapped_user: typing.Union[WrappedUser, typing.Dict],
        idempotency_key: typing.Optional[str] = None,
    ) -> Response[WrappedUser]:
        """Create a user, returning a `Response`.
        Please note that you must provide the user's email or the external id so MagicBell can uniquely identify the user.
        The external id, if provided, must be unique to the user.
        """
        response = await self.client.post(
            "/users",
            headers=self.configuration.get_general_headers(idempotency_key=idempotency_key),
            content=build_request_content(wrapped_user),
        )
        return build_response(response=response, out_type=WrappedUser)

    async def update_user(
        self,
        user_id: str,
        wrapped_user: typing.Union[WrappedUser, typing.Dict],
        idempotency_key: typing.Optional[str] = None,
    ) -> WrappedUser:
        """Update a user by id, email or external_id, returning a `WrappedUser`.
        The user id is the MagicBell user id.
        Alternatively, provide an id like `email:theusersemail@example.com` or `external_id:theusersexternalid` as the user id.
        """
        return (await self.update_user_detailed(user_id, wrapped_user, idempotency_key)).parsed

    async def update_user_detailed(
        self,
        user_id: str,
        wrapped_user: typing.Union[WrappedUser, typing.Dict],
        idempotency_key: typing.Optional[str] = None,
    ) -> Response[WrappedUser]:
        """Update a user by id, email or external_id, returning a `Response`.
        The user id is the MagicBell user id.
        Alternatively, provide an id like `email:theusersemail@example.com` or `external_id:theusersexternalid` as the user id.
        """
        url = f"/users/{user_id}"
        response = await self.client.put(
            url,
            headers=self.configuration.get_general_headers(idempotency_key=idempotency_key),
            content=build_request_content(wrapped_user),
        )
        return build_response(response=response, out_type=WrappedUser)

    async def delete_user(self, user_id: str, idempotency_key: typing.Optional[str] = None) -> None:
        """Delete a user by id, email or external_id.
        The user id is the MagicBell user id.
        Alternatively, provide an id like `email:theusersemail@example.com` or `external_id:theusersexternalid` as the user id.
        We will delete the user completely 7 days after.
        If the user makes a request to the API, the deletion will be canceled.
        This will happen when the notification inbox for this user is loaded in your app, for example.
        """
        await self.delete_user_detailed(user_id, idempotency_key)

    async def delete_user_detailed(
        self, user_id: str, idempotency_key: typing.Optional[str] = None
    ) -> Response[typing.Type[None]]:
        """Delete a user by id, email or external_id, returning a `Response`.
        The user id is the MagicBell user id.
        Alternatively, provide an id like `email:theusersemail@example.com` or `external_id:theusersexternalid` as the user id.
        We will delete the user completely 7 days after.
        If the user makes a request to the API, the deletion will be canceled.
        This will happen when the notification inbox for this user is loaded in your app, for example.
        """
        url = f"/users/{user_id}"
        response = await self.client.delete(
            url, headers=self.configuration.get_general_headers(idempotency_key=idempotency_key)
        )
        return build_response(response=response, out_type=None)



import typing

from ..model.channels import WrappedChannels
from ..model.response import Response
from ._base import BaseAPI
from ._parsing import build_response


class ChannelsAPI(BaseAPI):
    async def update_channels(self, wrapped_channels: WrappedChannels) -> None:
        """Update channel configuration for a project.

        Warnings
        --------
        This method is undocumented, proceed with caution.
        """
        await self.update_channels_detailed(wrapped_channels)

    async def update_channels_detailed(
        self, wrapped_channels: WrappedChannels, idempotency_key: typing.Optional[str] = None
    ) -> Response[typing.Type[None]]:
        """Update channel configuration for a project.

        Warnings
        --------
        This method is undocumented, proceed with caution.
        """
        response = await self.client.post(
            "/channels",
            content=wrapped_channels.model_dump_json(exclude_unset=True),
            headers=self.configuration.get_general_headers(idempotency_key=idempotency_key),
        )
        return build_response(response=response, out_type=None)

    async def get_channels(self) -> WrappedChannels:
        """Get channel configuration for a project.

        Warnings
        --------
        This method is undocumented, proceed with caution.
        """
        return (await self.get_channels_detailed()).parsed

    async def get_channels_detailed(self) -> Response[WrappedChannels]:
        """Get channel configuration for a project.

        Warnings
        --------
        This method is undocumented, proceed with caution.
        """
        response = await self.client.get(
            "/channels", headers=self.configuration.get_general_headers()
        )
        return build_response(response=response, out_type=WrappedChannels)



"""Defines a `ProjectAPI` class to interact with projects in MagicBell."""
import typing

from ..model.project import WrappedProject, WrappedProjectInput, WrappedProjects
from ..model.response import Response
from ._base import BaseAPI
from ._parsing import build_request_content, build_response


class ProjectAPI(BaseAPI):
    """Magicbell API methods for projects."""

    async def list_projects(self, workspace_id: int) -> WrappedProjects:
        """List all projects in a workspace, returning `WrappedProjects`."""
        return (await self.list_projects_detailed(workspace_id)).parsed

    async def list_projects_detailed(self, workspace_id: int) -> Response[WrappedProjects]:
        """List all projects in a workspace, returning a `Response`."""
        url = f"/workspaces/{workspace_id}/projects"
        response = await self.client.get(url, headers=self.configuration.get_user_headers())
        return build_response(response=response, out_type=WrappedProjects)

    async def get_project(self, workspace_id: int, project_id: int) -> WrappedProject:
        """Get a project, returning `WrappedProject`."""
        return (await self.get_project_detailed(workspace_id, project_id)).parsed

    async def get_project_detailed(
        self, workspace_id: int, project_id: int
    ) -> Response[WrappedProject]:
        """Get a project, returning a `Response`."""
        url = f"/workspaces/{workspace_id}/projects/{project_id}"
        response = await self.client.get(url, headers=self.configuration.get_user_headers())
        return build_response(response=response, out_type=WrappedProject)

    async def create_project(
        self, workspace_id: int, project_input: typing.Union[WrappedProjectInput, typing.Dict]
    ) -> WrappedProject:
        """Create a project, returning `WrappedProject`."""
        return (await self.create_project_detailed(workspace_id, project_input)).parsed

    async def create_project_detailed(
        self, workspace_id: int, project_input: typing.Union[WrappedProjectInput, typing.Dict]
    ) -> Response[WrappedProject]:
        """Create a project, returning a `Response`."""
        url = f"/workspaces/{workspace_id}/projects"
        response = await self.client.post(
            url,
            content=build_request_content(project_input),
            headers=self.configuration.get_user_headers(),
        )
        return build_response(response=response, out_type=WrappedProject)

    async def update_project(
        self,
        workspace_id: int,
        project_id: int,
        project_input: typing.Union[WrappedProjectInput, typing.Dict],
    ) -> WrappedProject:
        """Update a project, returning `WrappedProject`."""
        return (await self.update_project_detailed(workspace_id, project_id, project_input)).parsed

    async def update_project_detailed(
        self,
        workspace_id: int,
        project_id: int,
        project_input: typing.Union[WrappedProjectInput, typing.Dict],
    ) -> Response[WrappedProject]:
        """Update a project, returning a `Response`."""
        url = f"/workspaces/{workspace_id}/projects/{project_id}"
        response = await self.client.put(
            url,
            content=build_request_content(project_input),
            headers=self.configuration.get_user_headers(),
        )
        return build_response(response=response, out_type=WrappedProject)

    async def delete_project(self, workspace_id: int, project_id: int) -> None:
        """Delete a project."""
        await self.delete_project_detailed(workspace_id, project_id)

    async def delete_project_detailed(
        self, workspace_id: int, project_id: int
    ) -> Response[typing.Type[None]]:
        """Delete a project, returning a `Response`."""
        url = f"/workspaces/{workspace_id}/projects/{project_id}"
        response = await self.client.delete(url, headers=self.configuration.get_user_headers())
        return build_response(response=response, out_type=None)



import typing

from ..errors import MagicBellConfigurationError
from ..model import Response
from ._base import BaseAPI
from ._parsing import build_response


class GraphQLAPI(BaseAPI):
    """Perform GraphQL queries and mutations against the MagicBell GraphQL API.
    https://www.magicbell.com/docs/graphql-api/overview
    """

    async def query(
        self, query: str, variables: typing.Optional[typing.Dict] = None
    ) -> typing.Dict:
        """Query the GraphQL API, sends the X-MAGICBELL-API-KEY and X-MAGICBELL-SECRET-KEY headers.

        Use `user_query` if you want to send the X-MAGICBELL-API-KEY and X-MAGICBELL-USER-HMAC headers.
        """
        return (await self.query_detailed(query=query, variables=variables)).json_content()

    async def query_detailed(
        self, query: str, variables: typing.Optional[typing.Dict] = None
    ) -> Response[typing.Type[None]]:
        """Query the GraphQL API, sends the X-MAGICBELL-API-KEY and X-MAGICBELL-SECRET-KEY headers.

        Use `user_query` if you want to send the X-MAGICBELL-API-KEY and X-MAGICBELL-USER-HMAC headers.
        """
        return await self._query(
            headers=self.configuration.get_general_headers(), query=query, variables=variables
        )

    async def user_query(
        self,
        query: str,
        user_hmac: str,
        user_external_id: typing.Optional[str] = None,
        user_email: typing.Optional[str] = None,
        variables: typing.Optional[typing.Dict] = None,
    ) -> typing.Dict:
        """Query the GraphQL API as a user, sends the X-MAGICBELL-API-KEY and X-MAGICBELL-USER-HMAC headers.
        Must specify either `user_external_id` or `user_email`.
        """
        return (
            await self.user_query_detailed(
                query=query,
                user_hmac=user_hmac,
                user_external_id=user_external_id,
                user_email=user_email,
                variables=variables,
            )
        ).json_content()

    async def user_query_detailed(
        self,
        query: str,
        user_hmac: str,
        user_external_id: typing.Optional[str] = None,
        user_email: typing.Optional[str] = None,
        variables: typing.Optional[typing.Dict] = None,
    ) -> Response[typing.Type[None]]:
        """Query the GraphQL API as a user, sends the X-MAGICBELL-API-KEY and X-MAGICBELL-USER-HMAC headers.
        Must specify either `user_external_id` or `user_email`.
        """
        if not user_external_id and not user_email:
            raise MagicBellConfigurationError(
                "Either user_external_id or user_email must be provided"
            )

        headers = {
            "X-MAGICBELL-USER-HMAC": user_hmac,
            "X-MAGICBELL-API-KEY": self.configuration.api_key,
            **self.configuration.get_base_headers(),
        }
        if user_external_id:
            headers["X-MAGICBELL-USER-EXTERNAL-ID"] = user_external_id
        if user_email:
            headers["X-MAGICBELL-USER-EMAIL"] = user_email

        return await self._query(headers=headers, query=query, variables=variables)

    async def _query(
        self,
        headers: typing.Dict[str, str],
        query: str,
        variables: typing.Optional[typing.Dict] = None,
    ) -> Response:
        """Query the GraphQL API."""
        response = await self.client.post(
            "/graphql",
            json={"query": query, "variables": variables},
            headers=headers,
        )
        return build_response(response=response, out_type=None)



import typing

import orjson

from ._base import BaseModel

ResponseBodyT = typing.TypeVar("ResponseBodyT", bound=BaseModel)


class Response(BaseModel, typing.Generic[ResponseBodyT]):
    status_code: int
    content: bytes
    headers: typing.MutableMapping[str, str]
    parsed: typing.Optional[ResponseBodyT] = None

    def json_content(self) -> typing.Any:
        """Load `self.content` as JSON and return the parsed object"""
        return orjson.loads(self.content)



import typing

from ._base import BaseModel


class ChannelConfiguration(BaseModel):
    providers: typing.Dict


class Channel(BaseModel):
    slug: str
    configuration: typing.Optional[ChannelConfiguration] = None


class WrappedChannels(BaseModel):
    channels: typing.List[Channel]



from ._version import __version__  # noqa: F401
from .client import MagicBell  # noqa: F401
from .model import (  # noqa: F401
    Channel,
    ChannelConfiguration,
    ChannelOverrides,
    CreatedNotificationBroadcast,
    Notification,
    NotificationChannelsOverrides,
    NotificationOverrides,
    NotificationProvidersOverrides,
    Project,
    ProjectInput,
    PushSubscription,
    Recipient,
    User,
    WrappedChannels,
    WrappedCreatedNotificationBroadcast,
    WrappedNotification,
    WrappedProject,
    WrappedProjectInput,
    WrappedProjects,
    WrappedPushSubscription,
    WrappedUser,
)



import typing

import httpx

from .api import ChannelsAPI, GraphQLAPI, ProjectAPI, PushSubscriptionAPI, RealtimeAPI, UserAPI
from .configuration import Configuration


class MagicBell:
    """Central class for interacting with the MagicBell API.
    To be used as an asynchronous context manager.
    If an asynchronous context manager isn't feasible, then the `connect` and `disconnect` methods can be used.

    Examples
    --------

    >>> async with MagicBell() as mb:
    >>>    projects = (await mb.projects.list_projects()).projects
    >>>    for project in projects:
    >>>        print(project.id, project.name, project.api_key)
    >>> # OR: use an explicit config
    >>> async with MagicBell(Configuration(api_key="my-api-key", api_secret="my-secret")) as mb:
    >>>    ...
    """

    configuration: Configuration
    http_client: httpx.AsyncClient
    projects: ProjectAPI
    realtime: RealtimeAPI
    channels: ChannelsAPI
    graphql: GraphQLAPI

    def __init__(
        self,
        configuration: typing.Optional[Configuration] = None,
        http_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        """Create a new MagicBell instance.

        Parameters
        ----------
        configuration : Configuration, optional
            Configuration to use for the API.
            If not provided, a default configuration will be used.
            The default configuration will try to load configuration from environment variables using the `MAGICBELL_` prefix.
        http_client : httpx.AsyncClient, optional
            HTTP client to use for API requests. This is generally used for testing.
        """

        self.configuration = configuration or Configuration()
        self.http_client = http_client or httpx.AsyncClient(
            base_url=self.configuration.api_url,
            timeout=httpx.Timeout(timeout=self.configuration.request_timeout_seconds),
        )
        self._is_unmanaged_http_client = http_client is not None

        self.projects = ProjectAPI(self.http_client, self.configuration)
        self.realtime = RealtimeAPI(self.http_client, self.configuration)
        self.users = UserAPI(self.http_client, self.configuration)
        self.push_subscriptions = PushSubscriptionAPI(self.http_client, self.configuration)
        self.channels = ChannelsAPI(self.http_client, self.configuration)
        self.graphql = GraphQLAPI(self.http_client, self.configuration)

    async def connect(self) -> None:
        """Connect to the MagicBell API.

        This method is called automatically when the async context is entered.
        """
        if not self._is_unmanaged_http_client:
            await self.http_client.__aenter__()

    async def disconnect(self, exc_type=None, exc_val=None, exc_tb=None) -> None:
        """Disconnect from the MagicBell API.

        This method is called automatically when the async context is exited.
        """
        if not self._is_unmanaged_http_client:
            await self.http_client.__aexit__(exc_type, exc_val, exc_tb)

    async def __aenter__(self):
        await self.connect()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.disconnect(exc_type, exc_val, exc_tb)



import typing

from pydantic import Field

from ._base import BaseModel


class Recipient(BaseModel):
    """Represents a recipient in MagicBell."""

    email: typing.Optional[str] = Field(None, description="The user's email")
    external_id: typing.Optional[str] = Field(
        description="A unique string that MagicBell can utilize to uniquely identify the user. "
        "We recommend setting this attribute to the ID of the user in your database. "
        "Provide the external id if the user's email is unavailable.",
        default=None,
    )
    matches: typing.Optional[str] = Field(
        description="An SQL-like expression to match users by their stored attributes. "
        'Set it to "*" to send a notification to all users.',
        default=None,
    )


class ChannelOverrides(BaseModel):
    title: typing.Optional[str] = Field(None, description="Overridden title for this channel")
    content: typing.Optional[str] = Field(None, description="Overridden content for this channel")
    action_url: typing.Optional[str] = Field(
        None, description="Overridden action URL for this channel"
    )


class NotificationChannelsOverrides(BaseModel):
    in_app: typing.Optional[ChannelOverrides] = None
    email: typing.Optional[ChannelOverrides] = None
    web_push: typing.Optional[ChannelOverrides] = None
    mobile_push: typing.Optional[ChannelOverrides] = None


class NotificationProvidersOverrides(BaseModel):
    sendgrid: typing.Optional[typing.Dict[typing.Any, typing.Any]] = Field(
        description="Set of key-value pairs that you can pass on to Sendgrid. "
        "Applied only if it is configured for your project.",
        default=None,
    )
    mailgun: typing.Optional[typing.Dict[typing.Any, typing.Any]] = Field(
        description="Set of key-value pairs that you can pass on to Mailgun. "
        "Applied only if it is configured for your project.",
        default=None,
    )
    postmark: typing.Optional[typing.Dict[typing.Any, typing.Any]] = Field(
        description="Set of key-value pairs that you can pass on to Postmark. "
        "Applied only if it is configured for your project.",
        default=None,
    )
    ios: typing.Optional[typing.Dict[typing.Any, typing.Any]] = Field(
        description="Set of key-value pairs that you can pass on to APNS. "
        "Applied only if it is configured for your project.",
        default=None,
    )
    android: typing.Optional[typing.Dict[typing.Any, typing.Any]] = Field(
        description="Set of key-value pairs that you can pass on to FCM. "
        "Applied only if it is configured for your project.",
        default=None,
    )


class NotificationOverrides(BaseModel):
    channels: typing.Optional[NotificationChannelsOverrides] = Field(
        description="Overrides for specific channels", default=None
    )
    providers: typing.Optional[NotificationProvidersOverrides] = Field(
        description="Overrides for specific providers (Sendgrid, Postmark, APNs, etc)", default=None
    )


class Notification(BaseModel):
    title: str = Field(description="Title of the notification.")
    category: typing.Optional[str] = Field(
        description="Category the notification belongs to. "
        "This is useful to allow users to set their preferences.",
        default=None,
    )
    content: typing.Optional[str] = Field(
        description="Content of the notification. "
        "If you provide HTML content, the notification inbox will render it correctly.",
        default=None,
    )
    action_url: typing.Optional[str] = Field(
        description="A URL to redirect the user to when they click the notification in their notification inbox.",  # noqa: E501
        default=None,
    )
    custom_attributes: typing.Optional[typing.Dict[str, typing.Any]] = Field(
        description="Set of key-value pairs that you can attach to a notification. "
        "It accepts objects for the value of a key. "
        "You can use it to share data between channels or to render a custom UI.",
        default=None,
    )
    topic: typing.Optional[str] = Field(
        description="Topic the notification belongs to. This is useful to create threads.",
        default=None,
    )
    recipients: typing.List[Recipient] = Field(
        min_length=1,
        max_length=1000,
        description="Users to send the notification to. "
        "You can specify up to 1000 users at once. Use matches to send a notification to everyone",
    )
    overrides: typing.Optional[NotificationOverrides] = Field(
        description="Optional overrides to configure notifications per target destination",
        default=None,
    )


class WrappedNotification(BaseModel):
    broadcast: Notification


class CreatedNotificationBroadcast(BaseModel):
    id: str


class WrappedCreatedNotificationBroadcast(BaseModel):
    broadcast: CreatedNotificationBroadcast



import httpx

from belfry_magicbell.configuration import Configuration


class BaseAPI:
    def __init__(self, client: httpx.AsyncClient, configuration: Configuration) -> None:
        self.client = client
        self.configuration = configuration



import typing

from pydantic import Field

from ._base import BaseModel


class User(BaseModel):
    id: typing.Optional[str] = Field(
        description="The user's MagicBell ID (readonly).", default=None
    )
    external_id: typing.Optional[str] = Field(
        description="A unique string that MagicBell can utilize to identify the user uniquely. "
        "We recommend setting this attribute to the ID of the user in your database. "
        "Provide the external id if the user's email is unavailable.",
        default=None,
    )
    email: typing.Optional[str] = Field(description="The user's email.", default=None)
    first_name: typing.Optional[str] = Field(description="The user's first name.", default=None)
    last_name: typing.Optional[str] = Field(description="The user's last name.", default=None)
    custom_attributes: typing.Optional[typing.Dict[str, typing.Any]] = Field(
        description="Any customer attributes that you'd like to associate with the user. "
        "You may want to use these attributes later when writing email templates, for example.",
        default=None,
    )


class WrappedUser(BaseModel):
    user: User



import typing

from ._base import BaseModel
from .workspace import Workspace


class Project(BaseModel):
    """Represents a project in MagicBell."""

    id: int
    name: str
    hmac_enabled: bool
    api_key: str
    api_secret: str
    workspace: Workspace


class WrappedProject(BaseModel):
    """Represents a project in MagicBell (wrapped)."""

    project: Project


class WrappedProjects(BaseModel):
    """Represents a list of projects in MagicBell (wrapped)."""

    projects: typing.List[Project]


class ProjectInput(BaseModel):
    """Fields for creating or updating a project."""

    name: str
    hmac_enabled: typing.Optional[bool] = None


class WrappedProjectInput(BaseModel):
    """Fields for creating or updating a project (wrapped)."""

    project: ProjectInput



import logging
import typing

from ..model.notification import WrappedCreatedNotificationBroadcast, WrappedNotification
from ..model.response import Response
from ._base import BaseAPI
from ._parsing import build_request_content, build_response

logger = logging.getLogger(__name__)


class RealtimeAPI(BaseAPI):
    """APIs to manage notifications in real-time"""

    async def create_notification(
        self,
        wrapped_notification: typing.Union[WrappedNotification, typing.Dict],
        idempotency_key: typing.Optional[str] = None,
    ) -> WrappedCreatedNotificationBroadcast:
        """Send a notification to one or multiple users, returning a `Notification`.
        Specify `idempotency_key` to prevent duplicate notifications.
        https://www.magicbell.com/docs/rest-api/idempotent-requests
        """
        return (
            await self.create_notification_detailed(wrapped_notification, idempotency_key)
        ).parsed

    async def create_notification_detailed(
        self,
        wrapped_notification: typing.Union[WrappedNotification, typing.Dict],
        idempotency_key: typing.Optional[str] = None,
    ) -> Response[WrappedCreatedNotificationBroadcast]:
        """Send a notification to one or multiple users, returning a `Response`.
        Specify `idempotency_key` to prevent duplicate notifications.
        https://www.magicbell.com/docs/rest-api/idempotent-requests
        """
        try:
            response = await self.client.post(
                "/broadcasts",
                headers=self.configuration.get_general_headers(idempotency_key=idempotency_key),
                content=build_request_content(wrapped_notification),
            )
            return build_response(response=response, out_type=WrappedCreatedNotificationBroadcast)
        except Exception as e:
            rc = response.content
            logger.warning(
                f"Error sending {wrapped_notification} to magicbell; response - {rc}", exc_info=True
            )
            raise e



import typing

import httpx


class BaseMagicBellError(Exception):
    """Base class for all MagicBell errors."""

    pass


class MagicBellConfigurationError(BaseMagicBellError):
    """Raised when there is an error in the configuration."""

    pass


class BaseMagicBellHTTPError(BaseMagicBellError):
    """Base class for all MagicBell HTTP errors."""

    def __init__(self, response: httpx.Response):
        """Create a new MagicBell HTTP error.

        Parameters
        ----------
        response : httpx.Response
            The HTTP response that caused the error.
        """

        self.response = response

    def __str__(self):
        error_lines = [
            f"({self.response.status_code})",
            f"Reason: {self.response.reason_phrase}",
        ]
        if self.response.headers:
            error_lines.append(f"HTTP response headers: {self.response.headers}")
        if self.response.content:
            error_lines.append(f"HTTP response body: {self.response.text}")
        return "\n".join(error_lines)

    @property
    def status_code(self) -> int:
        return self.response.status_code

    @property
    def content(self) -> bytes:
        return self.response.content

    def json(self) -> typing.Any:
        return self.response.json()


class MagicBellHTTPError(BaseMagicBellHTTPError):
    """Unknown/unhandled HTTP errors."""

    pass


class MagicBellHTTPClientError(MagicBellHTTPError):
    """HTTP client errors (400-499)."""

    pass


class MagicBellHTTPServerError(MagicBellHTTPError):
    """HTTP server errors (500-599)."""

    pass



import typing

from pydantic import Field

from ._base import BaseModel


class PushSubscription(BaseModel):
    id: typing.Optional[str] = Field(
        description="The push subscription's MagicBell ID (readonly).", default=None
    )
    device_token: str = Field(
        description="Token that identifies the device. This is usually generated automatically by your app once installed."
    )
    platform: str = Field(
        description="The platform where the device token was generated from. This value is used to determine the delivery mechanism for mobile push notifications. Either 'ios', 'android' or 'safari'."
    )
    app_bundle_id: typing.Optional[str] = Field(
        description="The bundle ID of your app. This value is used to determine the delivery mechanism for mobile push notifications based on your workflow so that you can link several mobile applications to one project.",
        default=None,
    )


class WrappedPushSubscription(BaseModel):
    push_subscription: PushSubscription



import typing

import httpx
import orjson
from pydantic import BaseModel

from belfry_magicbell import errors
from belfry_magicbell.model import Response, ResponseBodyT


def check_response(response: httpx.Response) -> None:
    if response.is_client_error:
        raise errors.MagicBellHTTPClientError(response)
    elif response.is_server_error:
        raise errors.MagicBellHTTPServerError(response)
    elif not response.is_success:
        raise errors.MagicBellHTTPError(response)


def build_response(
    *, response: httpx.Response, out_type: typing.Optional[typing.Type[ResponseBodyT]]
) -> Response[ResponseBodyT]:
    """Transform an `httpx.Response` into a `Response`.

    Raises an error if the response is not successful.
    """

    check_response(response)

    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=out_type.model_validate_json(response.content) if out_type else None,
    )


def build_request_content(
    content: typing.Union[BaseModel, typing.Dict]
) -> typing.Union[str, bytes]:
    """Transform a `BaseModel` or a `dict` into a `str` or `bytes` for use with an http request."""
    if isinstance(content, BaseModel):
        return content.model_dump_json(exclude_unset=True)
    return orjson.dumps(content)
